package com.tobesoft.xplatform.common.env.rule;

import org.apache.commons.digester.CallMethodRule;
import org.apache.commons.digester.Digester;
import org.apache.commons.digester.Rule;
import org.apache.commons.digester.RuleSetBase;
import org.apache.commons.digester.SetPropertiesRule;
import org.xml.sax.Attributes;

import com.tobesoft.xplatform.common.env.metadata.ComponentEvent;
import com.tobesoft.xplatform.common.env.metadata.ComponentMethod;
import com.tobesoft.xplatform.common.env.metadata.ComponentProperty;
import com.tobesoft.xplatform.common.env.metadata.ComponentMetaData;

/**
 * <p><code>Component</code> 구조의 변환 규칙을 정의한 XML을 분석하는 <code>RuleSet</code>이다.
 *
 * @author Lee KiEun
 */
public class ComponentMetaRuleSet extends RuleSetBase {

	/**
	 * 기본 생성자이다.
	 */
	public ComponentMetaRuleSet() {
		;
	}

	/**
	 * <code>Digester</code>에 <code>Rule</code>를 추가한다.
	 */
	public void addRuleInstances(Digester digester) {
		String packageName = "com.tobesoft.xplatform.common.env.metadata";

		digester.addObjectCreate("component-list", "java.util.HashMap");
		digester.addObjectCreate("component-list/component", packageName + ".ComponentMetaData");
		digester.addRule("component-list/component", new ComponentRule());
		Rule rule = new CallMethodRule(1, "put", 2);
		digester.addRule("component-list/component", rule);
		digester.addCallParam("component-list/component", 0, "type");
		digester.addCallParam("component-list/component", 1, true);
		  
		addRule(digester
				, "component-list/component/property-list/property"
				, packageName + ".ComponentProperty"
				, "putProperty"
				, new CompPropertyRule());
		
		addRule(digester
				, "component-list/component/style-list/property"
				, packageName + ".ComponentProperty"
				, "putProperty"
				, new CompPropertyRule());

		addRule(digester
				, "component-list/component/method-list/method"
				, packageName + ".ComponentMethod"
				, "putMethod"
				, new CompMethodRule());

		addRule(digester
				, "component-list/component/event-list/event"
				, packageName + ".ComponentEvent"
				, "putEvent"
				, new CompEventRule());
	}

	/* Digester에 Rule을 추가한다. */
	private void addRule(Digester digester, String pattern, String className, String methodName, Rule rule) {
		digester.addObjectCreate(pattern, className);
		digester.addRule(pattern, rule);
		digester.addSetNext(pattern, methodName);
	}
	
	class ComponentRule extends SetPropertiesRule {

		public ComponentRule() {
			;
		}

		public void begin(Attributes attributes) throws Exception {
			super.begin(attributes);
			
			ComponentMetaData meta = (ComponentMetaData) digester.peek();
			meta.setType(attributes.getValue("type"));
			meta.setPackageName(attributes.getValue("packagename"));
			meta.setClassName(attributes.getValue("classname"));
			meta.setContainer(attributes.getValue("container").equals("true"));
			meta.setVisible(attributes.getValue("visible").equals("true"));
		}
	}
	
	class CompPropertyRule extends SetPropertiesRule {

		public CompPropertyRule() {
			;
		}

		public void begin(Attributes attributes) throws Exception {
			super.begin(attributes);
			
			ComponentProperty prop = (ComponentProperty) digester.peek();
			prop.setName(attributes.getValue("name"));
			prop.setType(attributes.getValue("type"));
			prop.setClassname(attributes.getValue("classname"));
		}
	}
	
	class CompMethodRule extends SetPropertiesRule {

		public CompMethodRule() {
			;
		}

		public void begin(Attributes attributes) throws Exception {
			super.begin(attributes);
			
			ComponentMethod method = (ComponentMethod) digester.peek();
			method.setName(attributes.getValue("name"));
		}
	}
	
	class CompEventRule extends SetPropertiesRule {

		public CompEventRule() {
			;
		}

		public void begin(Attributes attributes) throws Exception {
			super.begin(attributes);
			
			ComponentEvent event = (ComponentEvent) digester.peek();
			event.setName(attributes.getValue("name"));
		}
	}
}
